# @summary Installs the CIS Assessor.
#
# @param allow_insecure
#   Enables skipping of certificate validation when the scanner_source parameter is specified.
#
# @param manage_chocolatey
#   Manage chocolatey in this module for installing packages and allows for 7zip to install with archive,
#   requires chocolatey as the package provider
#
# @param linux_manage_unzip
#   Determines if this module should manage the installation of unzip on Linux
#
# @param win_manage_7zip
#   Determines if this module should manage the installation of 7zip on windows, requires chocolatey as the package provider.
#
# @param manage_java
#   Determines if this module should manage the installation of java
#   Note: This param is DEPRECATED and will be removed in a subsequent version.
#
# @param scanner_source
#   The URL to the location of a specific Assessor distribution within comply
#   This may be specified as a source attribute of a [file resource](https://puppet.com/docs/puppet/latest/types/file.html#file-attribute-source)
#
#   In the Puppet Comply application config, if you specified "We want to access the assessor through our ingress controller",
#   then set this value to https://`PE TLS FQDN`/assessor, otherwise set this value to https://`Comply Hostname`:30303/assessor
#
# @param scanner_source_linux
#   The URL to the location of a specific linux bundle should the user wish to host outside of comply.
#
# @param scanner_source_mac
#   The URL to the location of a specific mac bundle should the user wish to host outside of comply.
#
# @param scanner_source_windows
#   The URL to the location of a specific windows bundle should the user wish to host outside of comply.
#
# @param scanner_checksum_linux
#   Specify the sha256 checksum of the Linux Assessor archive provided in `scanner_source`.
#   Defaults to the checksum of the respective version of the scanner as part of Comply deployed in Puppet Application manager.
#
# @param scanner_checksum_mac
#   Specify the sha256 checksum of the Mac Assessor archive provided in `scanner_source`.
#   Defaults to the checksum of the respective version of the scanner as part of Comply deployed in Puppet Application manager.
#
# @param scanner_checksum_windows
#   Specify the sha256 checksum of the Windows Assessor archive provided in `scanner_source`.
#   Defaults to the checksum of the respective version of the scanner as part of Comply deployed in Puppet Application manager.
#
# @param scanner_version
#   Specify the version of the Assessor archive provided in `scanner_source`.
#   Defaults to the respective version of the scanner as part of Comply deployed in Puppet Application manager.
#
# @param use_mtls
#   Use mutual TLS authentication. Comply is configured to only accept requests from nodes with valid certificates issued with the same
#   CA cert configured in Replicated. Ensure you have configured Comply properly using certs generated on your PE instance.
#
# @param windows_manage_wget
#   Allow Comply to manage the wget package. Required for fetching the Assessor, requires chocolatey as the package provider.
#
# @param assessor_download_token
#   A bearer token to be used when downloading the Assessor.
#
# @param limits
#   Apply limits to the Assessor process, for example limit CPU usage.
#   The limits parameter is a hash with the following structure:
#   {
#     "systemd-run" => { # Linux only if /usr/bin/systemd-run is available
#       "CPUQuota" => 50 # Percentage of a single CPU core that the Assessor process is allowed to use, can be >100 for multiple cores.
#     },
#     "nice" => { # Linux and Mac only
#       "increment" => 10 # Increment the nice value of the Assessor process, can be negative.
#     }
#   }
#
# @example Use defaults
#   include comply
#
# @example Basic example
#   class { 'comply':
#     allow_insecure => false
#   }
#
class comply (
  Boolean $allow_insecure                                    = false,
  Boolean $linux_manage_unzip                                = true,
  Boolean $win_manage_7zip                                   = false,
  Boolean $manage_chocolatey                                 = true,
  Boolean $manage_java                                       = true,
  Optional[Stdlib::Filesource] $scanner_source               = undef,
  Optional[Stdlib::Filesource] $scanner_source_linux         = "${scanner_source}/linux",
  Optional[Stdlib::Filesource] $scanner_source_mac           = "${scanner_source}/mac",
  Optional[Stdlib::Filesource] $scanner_source_windows       = "${scanner_source}/windows",
  Optional[String] $scanner_checksum_linux             = 'af3826fb58bb5b2eab8af9ae31984d02d528f783b14254354824da6353366b44',
  Optional[String] $scanner_checksum_mac               = '79fd572025e27dbc2512d3d40b647d3d8fe338ddec85c2b455765fa6de155a54',
  Optional[String] $scanner_checksum_windows           = '7d9fe127362a1d2b6791aa6f50a6064f5e9f202ff87dac64bdfa1c1b63b6cd4d',
  Optional[Pattern[/\d+\.\d+\.\d+/]] $scanner_version        = '4.41.0',
  Boolean $use_mtls                                          = true,
  Boolean $windows_manage_wget                               = true,
  Optional[String] $assessor_download_token                  = '',
  Optional[Hash[String[1],Hash[String[1], Integer]]] $limits = undef,
) {
  include comply::scanners::ciscat
}
