#!/opt/puppetlabs/puppet/bin/ruby
# frozen_string_literal: true

require 'puppet'
require 'open3'
require 'json'

begin
  require_relative '../../ruby_task_helper/files/task_helper.rb'
  require_relative '../lib/task_helper'
rescue LoadError
  # include location for unit tests
  require 'fixtures/modules/ruby_task_helper/files/task_helper.rb'
  require 'fixtures/modules/comply/lib/task_helper'
end

# Ciscat scan task
class CiscatClass < TaskHelper
  def clean_reports(result_file_matcher, result_file)
    # remove existing scan result files
    Dir.glob(result_file_matcher).each { |file| File.delete(file) }
    FileUtils.rm_f(result_file)
  rescue Errno::ENOENT
    # do nothing, this is fine
  end

  def task(comply_server: nil, comply_port: nil, benchmark: nil, profile: nil, scan_hash: nil, custom_profile_id: nil,
           scan_type: 'ad hoc', debug: false, ssl_verify_mode: 'peer', scan_id: nil, auth_token: nil,
           scan_report_upload_path: '/api/v3/core/store/report', report_compression_mode: nil, **_kwargs)
    if (benchmark.nil? && scan_hash.nil?) || (!benchmark.nil? && !scan_hash.nil?)
      raise TaskHelper::Error.new('Either scan_hash OR benchmark must be set exclusively',
                                  'ciscatclass/cannot_parse_scan_hash',
                                  status: 1,
                                  err: scan_hash)
    end
    install_path, result_path = return_install_directories

    run_debug = if debug
                  '--debug -html'
                else
                  ''
                end
    # use the hash to get the profile and benchmark
    certname = run_puppet(['config', 'print', 'certname']).downcase
    if !scan_hash.nil?
      # parse the scan_hash string information to json
      begin
        parsed_hash = JSON.parse(scan_hash)
      rescue
        raise TaskHelper::Error.new("Cannot parse scan_hash #{scan_hash}",
                                    'ciscatclass/cannot_parse_scan_hash',
                                    status: 1,
                                    err: scan_hash)
      end
      # the following takes the hash and converts all keys or symbols to strings for lookup simplicity
      # nicer ways to do it in newer ruby versions or rails, but constrained to this while we support older ruby
      parsed_hash = parsed_hash.each_with_object({}) { |(k, v), memo| memo[k.to_s] = v; }
      # convert hostnames/keys to lowercase, looped this way for ruby 2.4
      lower_parsed_hash = {}
      parsed_hash.each_pair do |key, value|
        lower_parsed_hash[key.downcase] = value
      end
      parsed_hash = lower_parsed_hash
      benchmark_profile = parsed_hash[certname]
      if benchmark_profile.nil?
        raise TaskHelper::Error.new("No match for host #{certname}",
                                    'ciscatclass/hosts_doesnt_match_certname',
                                    status: -1, # jc_status does not return
                                    err: parsed_hash)
      end
      benchmark_file = benchmark_profile['benchmark']
      profile_option = '-p ' + benchmark_profile['profile']
      custom_profile_id = benchmark_profile['customProfileID']
    else
      benchmark_file = benchmark
      profile_option = if profile.nil?
                         ''
                       else
                         "-p #{profile}"
                       end
    end

    # remove existing scan result files
    result_file_matcher = File.join(result_path, 'puppet-compliance*.xml')
    debug_file_matcher = File.join(result_path, 'puppet-compliance*.html')
    result_file = File.join(result_path, 'xccdf-results.xml')
    clean_reports(result_file_matcher, result_file)

    # java required for linux/mac, embedded by CISCAT for windows
    os_facts = run_puppet(['facts', 'show', 'kernel'])
    begin
      os_type = JSON.parse(os_facts)['kernel']
    rescue
      raise TaskHelper::Error.new('Unable to determine OS',
                                    'ciscatclass/cannot_parse_facts',
                                    status: 1,
                                    err: os_facts)
    end
    java_path = if os_type == 'windows'
                  "#{install_path}jre/bin/java.exe"
                else
                  "#{install_path}jre/bin/java"
                end

    unless File.file?("#{install_path}benchmarks/#{benchmark_file}")
      raise TaskHelper::Error.new("Benchmark files not found #{install_path}benchmarks/#{benchmark_file}",
                                  'ciscatclass/benchmarkmissing',
                                  status: -1, # file does not return
                                  err: nil)
    end

    cmd = "#{java_path} -Xmx2048M -jar #{install_path}Assessor-CLI.jar -q #{run_debug} -rp puppet-compliance -rd #{result_path} #{profile_option} -b #{install_path}benchmarks/#{benchmark_file}"
    cmd = "#{cmd} -D ciscat.license.filepath=#{install_path}license/license.xml"

    assessor_limits_fact = run_puppet(['facts', 'show', 'assessor_limits'])
    begin
      assessor_limits_json = JSON.parse(assessor_limits_fact)['assessor_limits']
    rescue
      raise TaskHelper::Error.new('Unable to determine assessor_limits',
                                  'ciscatclass/cannot_parse_facts',
                                  status: 1,
                                  err: assessor_limits_fact)
    end

    if assessor_limits_json['systemd-run'] && File.file?('/usr/bin/systemd-run')
      assessor_limits_json['systemd-run'].each_pair do |key, value|
        if key.casecmp('cpuquota').zero? && value.is_a?(Integer)
          cmd = "/usr/bin/systemd-run -q --scope -p CPUQuota=#{value}% " + cmd
        end
      end
    elsif assessor_limits_json['nice'] && File.file?('/usr/bin/nice')
      assessor_limits_json['nice'].each_pair do |key, value|
        if key.casecmp('increment').zero? && value.is_a?(Integer)
          cmd = "/usr/bin/nice -n #{value} " + cmd
        end
      end
    end

    stdout, stderr, status = Open3.capture3(cmd, chdir: install_path)
    begin
      Dir.glob(result_file_matcher).each { |file| File.rename(file, result_file) }
    rescue Errno::ENOENT => e
      raise TaskHelper::Error.new("Unable to locate scan report '#{result_file}'",
                                  'ciscatclass/missingreport',
                                  status: status.exitstatus,
                                  cmd_ran: cmd,
                                  err: e.message)
    end
    if debug
      begin
        # figure out what the latest debug file created was
        debug_file = Dir.glob(debug_file_matcher).max_by { |f| File.mtime(f) }
        debug_file = File.basename(debug_file)
      rescue Errno::ENOENT
        # If the debug file didnt get created, we want to continue, getting the report is more important,
        # if we get here the report exists, so deliver that.
      end
    end
    unless status.exitstatus.zero?
      raise TaskHelper::Error.new("Scan did not complete successfully '#{cmd}', '#{stderr}'",
                                  'ciscatclass/noscan',
                                  status: status.exitstatus,
                                  cmd_ran: cmd,
                                  err: stderr)
    end

    # upload to comply server
    unless comply_server == 'nil'
      response = upload_scan_result(result_file: result_file, format: 'ciscat', comply_server: comply_server,
                                    comply_port: comply_port, custom_profile_id: custom_profile_id,
                                    scan_type: scan_type, certname: certname, ssl_verify_mode: ssl_verify_mode,
                                    scan_id: scan_id, auth_token: auth_token,
                                    scan_report_upload_path: scan_report_upload_path, compression_mode: report_compression_mode)
      if response.code != '200'
        raise TaskHelper::Error.new("Report upload failed #{response.code}",
                                    'ciscatclass/upload_failed',
                                    status: response.code,
                                    err: response.value)
      end
    end
    clean_reports(result_file_matcher, result_file) unless debug # don't remove report if debug enabled
    result = {
      status: status.exitstatus,
      scan_cmd: cmd,
      stdout: stdout,
      result_file: result_file,
      scan_id: scan_id,
    }
    result[:debug_file] = debug_file if debug
    result
  end
end

if $PROGRAM_NAME == __FILE__
  CiscatClass.run
end
